@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit.c,v 1.133 2006/03/03 16:31:17 paolo Exp $
+ * $Id: gedit.c,v 1.138 2006/04/20 10:40:46 sfre Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -81,7 +81,7 @@ static const GOptionEntry options [] =
 	  N_("Create a new document in an existing instance of gedit"), NULL },
 
 	{ G_OPTION_REMAINING, '\0', 0, G_OPTION_ARG_FILENAME_ARRAY, &remaining_args,
-	  NULL, NULL }, /* collects file arguments */
+	  NULL, N_("[FILE...]") }, /* collects file arguments */
 
 	{NULL}
 };
@@ -169,6 +169,29 @@ get_startup_timestamp ()
 	return (retval > 0) ? retval : 0;
 }
 
+static GdkDisplay *
+display_open_if_needed (const gchar *name)
+{
+	GSList *displays;
+	GSList *l;
+	GdkDisplay *display = NULL;
+
+	displays = gdk_display_manager_list_displays (gdk_display_manager_get ());
+
+	for (l = displays; l != NULL; l = l->next)
+	{
+		if (strcmp (gdk_display_get_name ((GdkDisplay *) l->data), name) == 0)
+		{
+			display = l->data;
+			break;
+		}
+	}
+
+	g_slist_free (displays);
+
+	return display != NULL ? display : gdk_display_open (name);
+}
+
 /* serverside */
 static void
 on_message_received (const char *message,
@@ -198,7 +221,7 @@ on_message_received (const char *message,
 	screen_number = atoi (params[2]);
 	workspace = atoi (params[3]);
 
-	display = gdk_display_open (display_name);
+	display = display_open_if_needed (display_name);
 	screen = gdk_display_get_screen (display, screen_number);
 	g_free (display_name);
 
@@ -418,6 +441,7 @@ main (int argc, char *argv[])
 	setlocale (LC_ALL, "");
 
 	bindtextdomain (GETTEXT_PACKAGE, GEDIT_LOCALEDIR);
+	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
 	startup_timestamp = get_startup_timestamp();
@@ -425,7 +449,7 @@ main (int argc, char *argv[])
 	gedit_debug_message (DEBUG_APP, "Run gnome_program_init");
 
 	/* Setup command line options */
-	context = g_option_context_new ("[FILE...]"); /* Should be translated when out of string freeze */
+	context = g_option_context_new (_("- Edit text files"));
 	g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
 	
 	/* Initialize gnome program */
@@ -439,10 +463,6 @@ main (int argc, char *argv[])
 
 	gedit_debug_message (DEBUG_APP, "Done gnome_program_init");
 
-	/* Must be called after gnome_program_init to avoid problem with the
-         * translation of --help messages */
-	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-
 	gedit_debug_message (DEBUG_APP, "Create bacon connection");
 
 	connection = bacon_message_connection_new ("gedit");
@@ -480,6 +500,9 @@ main (int argc, char *argv[])
 
 	gedit_debug_message (DEBUG_APP, "Set icon");
 	
+	gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (),
+					   GEDIT_ICONDIR);
+
 	/* Set default icon */
 	gtk_window_set_default_icon_name ("text-editor");
 
@@ -557,3 +580,4 @@ main (int argc, char *argv[])
 
 	return 0;
 }
+
